/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "getview-function.h"
#include <sys/xattr.h>
#include <errno.h>
#include <iostream>



getview_function::getview_function ()
: ctl_function ()
{
}



bool getview_function::execute (int &argc, char *argv[])
{
  if (argc < 3)
  {
    help ();
    
    return false;
  }

  char buffer[4096];
  
  int size = lgetxattr (argv[2], "view", buffer, 4096);
  
  if (size < 0)
  {
    int err = errno;
  
    std::cerr << "Error retrieving view." << std::endl
              << "  Code : " << err << " (" << strerror (err) << ")" << std::endl;

    return false;
  }

  char *ptr = buffer;
  for (int loop = 0; loop < size; loop += strlen (ptr) + 1)
  {
    std::cout << ptr << std::endl;
    ptr = &buffer[loop];
  }

  return true;
}



void getview_function::help () const
{
  std::cerr << "Displays the view settings for a cvsfs-fuse mountpoint." << std::endl
            << std::endl
	    << "Command:" << std::endl
	    << "  cvsfsctl getview <mountpoint>" << std::endl;
}
