/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "condition.h"
#include <sys/time.h>
#include <errno.h>
#include "system-log.h"



condition::condition ()
{
  int ret = ::pthread_cond_init (&cond, 0);
  
  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not initialize condition. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }
}



bool condition::signal ()
{
  int ret = ::pthread_cond_signal (&cond);
  
  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not signal condition. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}



bool condition::broadcast ()
{
  int ret = ::pthread_cond_broadcast (&cond);
  
  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not broadcast condition. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}



bool condition::wait ()
{
  int ret = ::pthread_cond_wait (&cond, &fmutex);

  if (ret != 0)
  {
    system_log *log = system_log::instance ();

    log->error << "Can not wait for condition. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}



bool condition::wait (double seconds)
{
  struct timeval now;
  struct timespec timeout;

  gettimeofday (&now, 0);

  TIMEVAL_TO_TIMESPEC (&now, &timeout);
  timeout.tv_sec += static_cast<time_t> (seconds);

  int ret = ::pthread_cond_timedwait (&cond, &fmutex, &timeout);

  if ((ret != 0) && (ret != ETIMEDOUT))
  {
    system_log *log = system_log::instance ();

    log->error << "Can not wait for condition. Return code: " << ret << " (" << strerror (ret) << ")" << std::endl;
  }

  return ret == 0;
}
