/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "connected-socket.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <errno.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>	// RedHat 7.2 requires this to provide 'memcpy'



connected_socket::connected_socket(int fd, sockaddr_in & addr)
: sockfd (fd)
{
  ::memcpy (&address, &addr, sizeof (struct sockaddr_in));
}



connected_socket::~connected_socket()
{
  close ();
}



size_t connected_socket::send (const void *message, int length)
{
  int bytesSent;
  fd_set rset;
  struct timeval tv;

  FD_ZERO (&rset);
  FD_SET (sockfd, &rset);
  tv.tv_sec = 20;
  tv.tv_usec = 0;

  ::select (sockfd + 1, NULL, &rset, NULL, &tv);

  if (FD_ISSET (sockfd, &rset))
    bytesSent = ::send (sockfd, message, length, 0);
  else
    bytesSent = -1;

  return bytesSent;
}



int connected_socket::recv (void *buffer, int length)
{
  int bytesRecv;
  fd_set rset;
  struct timeval tv;

  FD_ZERO (&rset);
  FD_SET (sockfd, &rset);
  tv.tv_sec = 10;
  tv.tv_usec = 0;

  ::select (sockfd + 1, &rset, NULL, NULL, &tv);

  if (FD_ISSET (sockfd, &rset))
    bytesRecv = ::recv (sockfd, buffer, length, 0);
  else
    bytesRecv = -1;

  return bytesRecv;
}



bool connected_socket::close ()
{
  int result;
  
  result = ::shutdown (sockfd, SHUT_RDWR);

  if (result == -1) 
    return false;

  return true;
}



char *connected_socket::get_ip ()
{
  return ::inet_ntoa (address.sin_addr);
}
