/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __CVS_DIRECTORY_ENTRY_H__
#define __CVS_DIRECTORY_ENTRY_H__

#include <sys/stat.h>
#include <string>
#include <list>
#include <iostream>
#include "directory.h"



class cvs_directory_entry
{
  public:
    cvs_directory_entry (const std::string & = "", const std::string & = "", bool = false);
    cvs_directory_entry (const cvs_directory_entry &);
    ~cvs_directory_entry ();

//    cvs_directory_entry * clone ();

    cvs_directory_entry & operator = (const cvs_directory_entry &);

    std::ostream & operator << (std::ostream &);
    std::istream & operator >> (std::istream &);

    friend std::ostream & operator << (std::ostream &, cvs_directory_entry &);
    friend std::istream & operator >> (std::istream &, cvs_directory_entry &);

    const std::string & name () const { return data->cvs_filename; }
    const std::string & version () const { return data->cvs_fileversion; }
    void set_version (const std::string &version) { data->cvs_fileversion = version; }

    void set_attributes (const struct stat &);
    void add_entry (cvs_directory_entry &);
    void remove_entry (const std::string &);

    void cleanup_tree (time_t);

    void get_directory (directory *) const;
    bool get_entry (const std::string &, cvs_directory_entry &) const;
    bool get_deep_entry (const std::string &, cvs_directory_entry &) const;
    bool get_attributes (struct stat &) const;

    bool is_dir () const { return data->is_directory; }
    bool valid () const { return data->valid_attributes; }

    void erase_contents ();
    void invalidate () { if (!data->is_directory) data->valid_attributes = false; }

    void set_logfilename (const std::string &);
    const std::string &get_logfilename () const { return data->log_filename; }

    time_t get_scantime () const { return data->last_scanned; }
    void set_scantime (time_t t) { data->last_scanned = t; }

  private:
//    cvs_directory_entry ();	// forbid use of default constructor
  
    std::istream & read_string (std::string &, std::istream &);
    std::istream & read_bool (bool &, std::istream &);

//    void increment_refcount ();
//    void decrement_refcount ();

    typedef std::list<cvs_directory_entry> directory_list;

    class entry_data
    {
      public:
        entry_data (const std::string &, const std::string &, bool);

        int refcount;
        std::string cvs_filename;
        std::string cvs_fileversion;
        bool is_directory;
        bool valid_attributes;
        struct stat cvs_filestats;
        directory_list contents;
        std::string log_filename;
        time_t last_scanned;
    };

    struct entry_data *data;
};



#endif
