/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "cvs-loadtree-thread.h"
#include <sys/time.h>
#include "cvs-connection.h"
#include "cvs-command-tree.h"
#include "system-log.h"



cvs_loadtree_thread::cvs_loadtree_thread (cvs_connection &conn,
					  cvs_directory_entry &dir,
					  const std::string &lab)
: connection (conn.clone ()), root (dir), label (lab),
  last_update (0), last_duration (0), terminate (), terminate_completed ()
{
  // this thread runs detached
  set_detached ();
}



cvs_loadtree_thread::~cvs_loadtree_thread ()
{
  // must wait until thread terminates
  if (is_running ())
  {
    terminate.signal ();

    terminate_completed.wait ();
  }

  delete connection;
}



void cvs_loadtree_thread::run ()
{
  system_log *log = system_log::instance ();

  log->debug << "cvs_loadtree::run: thread started" << std::endl;

  while (!terminate.wait (last_duration * 10))	// minmal 10 seconds delay
  {
    cvs_command_tree command (root, label);

    if (connection->execute_command (command))
    {
      // these times are to be used to calculate the next scan time
      last_update = command.execution_start ();
      last_duration = command.execution_time () + 1;
    }
  }

  log->debug << "cvs_loadtree::run: thread stopped" << std::endl;

  terminate_completed.post ();
}
