/***************************************************************************
 *   Copyright (C) 2004 by Petric Frank                                    *
 *   pfrank@gmx.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "cvs-pserver-streambuf.h"

#include "connected-socket.h"
//#include "system-log.h"



cvs_pserver_streambuf::cvs_pserver_streambuf (connected_socket &file)
: std::streambuf (), fFile (file), fPos (0)
{
//  system_log *log = system_log::instance ();

//  log->debug << "cvs_pserver_streambuf::<constructor>: _M_in_cur = " << _M_in_cur << std::endl;

  // disable any buffers
  setg (0, 0, 0);
  setp (0, 0);

//  log->debug << "cvs_pserver_streambuf::<constructor>: _M_in_cur = " << _M_in_cur << std::endl;
}



cvs_pserver_streambuf::~cvs_pserver_streambuf ()
{
  if (fPos > 0)
    sync ();
}



int cvs_pserver_streambuf::overflow (int ch)
{
  if (ch != EOF)
  {
    if (fPos >= sizeof (fBuffer))
      if (sync () < 0)
        return EOF;

    fBuffer[fPos] = ch;
    ++fPos;

    if (ch == '\n')
      if (sync () < 0)
        return EOF;
  }

  return ch;
}



std::streambuf::int_type cvs_pserver_streambuf::underflow ()
{
//  system_log *log = system_log::instance ();
  char_type item;

//  log->debug << "cvs_pserver_streambuf::underflow: _M_in_cur = " << _M_in_cur << std::endl;

  if (fFile.recv (&item, 1) == 1)
  {
//    log->debug << "cvs_pserver_streambuf::underflow: item received: '" << item << "'" << std::endl;
    return traits_type::to_int_type (item);
  }

//  log->debug << "cvs_pserver_streambuf::underflow: nothing got" << std::endl;

  return std::streambuf::underflow ();
}



std::streambuf::int_type cvs_pserver_streambuf::uflow ()
{
  return underflow ();
}



int cvs_pserver_streambuf::sync ()
{
  bool result = (fFile.send (fBuffer, fPos) == fPos);

  fPos = 0;

  return result ? 0 : -1;
}
